"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const d3 = require("d3");
const axes = require("./axes");
class CompData {
    constructor() {
        this.cls = "";
    }
    classed(cls) {
        if (cls === undefined) {
            return this.cls;
        }
        else {
            this.cls = cls;
            return this;
        }
    }
}
exports.CompData = CompData;
class Value extends CompData {
    constructor(label, value) {
        super();
        this.lab = label;
        this.val = value;
    }
    plot(elem, x, format) {
        // TODO In d3 style this should probably join on this.val, then width will
        // be a function of d, and so will the text
        elem.append("g").classed("princ--value", true)
            .append("rect")
            .attr("y", "-0.5em")
            .attr("height", "1em")
            .attr("width", x(this.val));
        elem.append("g").classed("princ--name", true)
            .append("text").text(this.lab);
        elem.append("g").attr("transform", `translate(${x.range().reverse()[0]}, 0)`)
            .append("g").classed("princ--num", true)
            .append("text").text((_, i) => format(this.val, i));
    }
}
exports.Value = Value;
class Comparison {
    constructor(width) {
        this.wdth = width;
        this.data = [];
        this.cls = "princ--jeanluc-grey";
        // Bounds
        this.maxVal = 0.0;
        this.maxSet = false;
        this.scale = d3.scaleLinear();
        this.format = d3.format("");
        // Axis
        this.label = "";
        this.ticks = [];
        this.axisFormat = d3.format("");
    }
    value(label, value) {
        this.maxSet || (this.maxVal = Math.max(this.maxVal, value));
        const datum = new Value(label, value);
        this.data.push(datum);
        return datum;
    }
    width(width) {
        this.wdth = width;
        return this;
    }
    numberFormat(format) {
        this.format = format;
        return this;
    }
    max(max) {
        if (max === undefined) {
            return this.maxVal;
        }
        else {
            this.maxVal = max;
            this.maxSet = true;
            return this;
        }
    }
    axis({ label = this.label, ticks = this.ticks, format = this.axisFormat }) {
        this.label = label;
        this.ticks = ticks;
        this.axisFormat = format;
        return this;
    }
    /** Get or set classes for the data
     * @param cls If specified, set the classes to the space delimited string,
     * otherwise returns the current classes.
     */
    classed(cls) {
        if (cls === undefined) {
            return this.cls;
        }
        else {
            this.cls = cls;
            return this;
        }
    }
    plot(svgElement) {
        const svg = d3.select(svgElement).append("g").classed("princ--root", true).classed(this.cls, true);
        const x = this.scale.range([0, this.wdth]).domain([0, this.maxVal]);
        const dataGroup = svg.append("g");
        this.data.forEach((datum, i) => {
            const line = dataGroup.append("g")
                .classed("princ--item", true)
                .attr("style", `transform: translateY(${i}em);`)
                .append("g").classed(datum.classed(), true);
            datum.plot(line, x, this.format);
        });
        // axes
        const axisGroup = svg
            .append("g").classed("princ--axis", true)
            .append("g").attr("style", `transform: translateY(${this.data.length - 1}.5em);`);
        axes.xaxis(axisGroup, x, [0, this.maxVal].concat(this.ticks), this.axisFormat, this.label);
    }
}
exports.Comparison = Comparison;
function comparison(width = 162) {
    return new Comparison(width);
}
exports.comparison = comparison;
